@@ -46,7 +46,7 @@ islines.sf <- function(g1, g2) {
 #' lib_versions <- sf::sf_extSoftVersion()
 #' lib_versions
 #' # fails on some systems (with early versions of PROJ)
-#' if(lib_versions[3] >= "6.3.1") {
+#' if (lib_versions[3] >= "6.3.1") {
 #'   sl <- routes_fast_sf[2:4, ]
 #'   rsec <- gsection(sl)
 #'   length(rsec) # sections
@@ -114,23 +114,21 @@ lineLabels <- function(sl, attrib) {
 #' The function can be used to estimate the amount of transport 'flow' at the
 #' route segment level based on input datasets from routing services, for
 #' example linestring geometries created with the `route()` function.
-#'
-#' @param attrib A character vector corresponding to the variables in `sl$` on
-#'   which the function(s) will operate.
-#' @param fun The function(s) used to aggregate the grouped values (default:
-#'   sum). If length of `fun` is smaller than `attrib` then the functions are
-#'   repeated for subsequent attributes.
-#' @param na.zero Sets whether aggregated values with a value of zero are
-#'   removed.
-#' @param ... Arguments passed to `overline2`
-#' @inheritParams gsection
-#' @inheritParams overline2
+#' @param sl A spatial object representing routes on a transport network
+#' @param attrib character, column names in sl to be aggregated
+#' @param ncores integer, how many cores to use in parallel processing, default = 1
+#' @param simplify logical, if TRUE group final segments back into lines, default = TRUE
+#' @param regionalise integer, during simplification regonalisation is used if the number of segments exceeds this value
+#' @param quiet Should the the function omit messages? `NULL` by default,
+#' which means the output will only be shown if `sl` has more than 1000 rows.
+#' @param fun Named list of functions to summaries the attributes by? `sum` is the default.
+#' `list(sum = sum, average = mean)` will summarise all `attrib`utes by sum and mean.
 #' @author Barry Rowlingson
 #' @references
 #' Morgan M and Lovelace R (2020). Travel flow aggregation: Nationally scalable methods
 #' for interactive and online visualisation of transport behaviour at the road network level.
 #' Environment and Planning B: Urban Analytics and City Science. July 2020.
-#' [doi:10.1177/2399808320942779](https://doi.org/10.1177/2399808320942779)
+#' \doi{10.1177/2399808320942779}.
 #'
 #' Rowlingson, B (2015). Overlaying lines and aggregating their values for overlapping
 #' segments. Reproducible question from <https://gis.stackexchange.com>. See
@@ -146,7 +144,9 @@ lineLabels <- function(sl, attrib) {
 #'
 #' - `ncores`, the number of cores to use in parallel processing
 #' - `simplify`, should the final segments be converted back into longer lines? The default
-#' setting.
+#' setting is `TRUE`. `simplify = FALSE` results in straight line segments consisting
+#' of only 2 vertices (the start and end point),
+#' resulting in a data frame with many more rows than the simplified results (see examples).
 #' - `regionalise` the threshold number of rows above which
 #' regionalisation is used (see details).
 #'
@@ -172,14 +172,20 @@ lineLabels <- function(sl, attrib) {
 #' @family rnet
 #' @export
 #' @examples
-#'
 #' sl <- routes_fast_sf[2:4, ]
-#' class(sl)
-#' class(sl$geometry)
-#' overline(sl = sl, attrib = "length")
-#' rnet_sf <- overline(sl = sl, attrib = "length", quiet = FALSE)
-#' plot(rnet_sf, lwd = rnet_sf$length / mean(rnet_sf$length))
-#'
+#' sl$All <- flowlines$All[2:4]
+#' rnet <- overline(sl = sl, attrib = "All")
+#' nrow(sl)
+#' nrow(rnet)
+#' plot(rnet)
+#' rnet_mean <- overline(sl, c("All", "av_incline"), fun = list(mean = mean, sum = sum))
+#' plot(rnet_mean, lwd = rnet_mean$All_sum / mean(rnet_mean$All_sum))
+#' rnet_sf_raw <- overline(sl, attrib = "length", simplify = FALSE)
+#' nrow(rnet_sf_raw)
+#' summary(n_vertices(rnet_sf_raw))
+#' plot(rnet_sf_raw)
+#' rnet_sf_raw$n <- 1:nrow(rnet_sf_raw)
+#' plot(rnet_sf_raw[10:25, ])
 #' # legacy implementation based on sp data
 #' # sl <- routes_fast[2:4, ]
 #' # rnet1 <- overline(sl = sl, attrib = "length")
@@ -188,24 +194,225 @@ lineLabels <- function(sl, attrib) {
 #' # plot(rnet2, lwd = rnet2$length / mean(rnet2$length))
 overline <- function(sl,
                      attrib,
-                     fun = sum,
-                     na.zero = FALSE,
-                     buff_dist = 0,
-                     ...) {
+                     ncores = 1,
+                     simplify = TRUE,
+                     regionalise = 1e5,
+                     quiet = ifelse(nrow(sl) < 1000, TRUE, FALSE),
+                     fun = sum) {
   UseMethod("overline")
 }
+#' Convert series of overlapping lines into a route network (new method)
+#'
+#' @description This function is intended as a replacement for overline() and is significantly faster
+#' especially on large datasets. However, it also uses more memory.
+#'
+#' @family rnet
+#' @author Malcolm Morgan
+#' @return An `sf` object representing a route network
 #' @export
-overline.sf <- function(sl, attrib, fun = sum, na.zero = FALSE, buff_dist = 0, ...) {
-  overline2(sl,
-    attrib,
-    ncores = 1,
-    simplify = TRUE,
-    regionalise = 1e5,
-    ...
-  )
+#' @rdname overline
+overline2 <-
+  function(sl,
+           attrib,
+           ncores = 1,
+           simplify = TRUE,
+           regionalise = 1e5,
+           quiet = ifelse(nrow(sl) < 1000, TRUE, FALSE),
+           fun = sum) {
+    if(as.character(unique(sf::st_geometry_type(sl))) == "MULTILINESTRING") {
+      message("Converting from MULTILINESTRING to LINESTRING")
+      sl <- sf::st_cast(sl, "LINESTRING")
+    }
+    if (!"sfc_LINESTRING" %in% class(sf::st_geometry(sl))) {
+      stop("Only LINESTRING is supported")
+    }
+    if (any(c("1", "2", "3", "4", "grid") %in% attrib)) {
+      stop("1, 2, 3, 4, grid are not a permitted column names, please rename that column")
+    }
+    sl <- sf::st_zm(sl)
+    sl <- sl[, attrib]
+    sl_crs <- sf::st_crs(sl)
+    if (!quiet) {
+      message(paste0(Sys.time(), " constructing segments"))
+    }
+    c1 <- sf::st_coordinates(sl)
+    sf::st_geometry(sl) <- NULL
+    l1 <- c1[, 3] # Get which line each point is part of
+    c1 <- c1[, 1:2]
+    l1_start <- duplicated(l1) # find the break points between lines
+    l1_start <- c(l1_start[2:length(l1)], FALSE)
+    c2 <- c1[2:nrow(c1), 1:2] # Create new coords offset by one row
+    c2 <- rbind(c2, c(NA, NA))
+    c2[nrow(c1), ] <- c(NA, NA)
+    c2[!l1_start, 1] <- NA
+    c2[!l1_start, 2] <- NA
+    c3 <- cbind(c1, c2) # make new matrix of start and end coords
+    rm(c1, c2)
+    c3 <- c3[!is.na(c3[, 3]), ]
+    sl <- sl[l1[l1_start], , drop = FALSE] # repeate attributes
+    rm(l1, l1_start)
+
+    # if (!quiet) {
+    #   message(paste0(Sys.time(), " transposing 'B to A' to 'A to B'"))
+    # }
+    attributes(c3)$dimnames <- NULL
+    c3 <- t(apply(c3, MARGIN = 1, FUN = function(y) {
+      if (y[1] != y[3]) {
+        if (y[1] > y[3]) {
+          c(y[3], y[4], y[1], y[2])
+        } else {
+          y
+        }
+      } else {
+        if (y[2] > y[4]) {
+          c(y[3], y[4], y[1], y[2])
+        } else {
+          y
+        }
+      }
+    }))
+
+    # if (!quiet) {
+    #   message(paste0(Sys.time(), " removing duplicates"))
+    # }
+    sl <- cbind(c3, sl)
+    rm(c3)
+
+    # browser()
+    # if(requireNamespace("data.table", quietly = TRUE)) {
+    #   sl = data.table::data.table(sl)
+    # }
+    slg <- dplyr::group_by_at(sl, c("1", "2", "3", "4"))
+    sls <- dplyr::ungroup(dplyr::summarise_all(slg, .funs = fun))
+    attrib <- names(sls)[5:ncol(sls)]
+    coords <- as.matrix(sls[, 1:4])
+    sl <- sls[, -c(1:4)]
+
+    # Make Geometry
+    if (!quiet) {
+      message(paste0(Sys.time(), " building geometry"))
+    }
+    sf::st_geometry(sl) <- sf::st_as_sfc(
+      if (requireNamespace("pbapply", quietly = TRUE)) {
+        pbapply::pblapply(1:nrow(coords), function(y) {
+          sf::st_linestring(matrix(coords[y, ], ncol = 2, byrow = T))
+        })
+      } else {
+        lapply(1:nrow(coords), function(y) {
+          sf::st_linestring(matrix(coords[y, ], ncol = 2, byrow = T))
+        })
+      },
+      crs = sl_crs
+    )
+    rm(coords)
+
+    # Recombine into fewer lines
+    if (simplify) {
+      if (!quiet) {
+        message(paste0(Sys.time(), " simplifying geometry"))
+      }
+      if (nrow(sl) > regionalise) {
+        message(paste0("large data detected, using regionalisation, nrow = ", nrow(sl)))
+        suppressWarnings(cents <- sf::st_centroid(sl))
+        grid <- sf::st_make_grid(cents, what = "polygons")
+        suppressWarnings(inter <- unlist(lapply(sf::st_intersects(cents, grid), `[[`, 1)))
+        sl$grid <- inter
+        rm(cents, grid, inter)
+        # split into a list of df by grid
+        sl <- dplyr::group_split(sl, grid)
+        message(paste0(Sys.time(), " regionalisation complete, aggregating flows"))
+        if (ncores > 1) {
+          cl <- parallel::makeCluster(ncores)
+          parallel::clusterExport(
+            cl = cl,
+            varlist = c("attrib"),
+            envir = environment()
+          )
+          parallel::clusterEvalQ(cl, {
+            library(sf)
+            # library(dplyr)
+          })
+          overlined_simple <- if (requireNamespace("pbapply", quietly = TRUE)) {
+            pbapply::pblapply(sl, function(y) {
+              y <- dplyr::group_by_at(y, attrib)
+              y <- dplyr::summarise(y, do_union = FALSE, .groups = "drop")
+            }, cl = cl)
+          } else {
+            lapply(sl, function(y) {
+              y <- dplyr::group_by_at(y, attrib)
+              y <- dplyr::summarise(y, do_union = FALSE, .groups = "drop")
+            })
+          }
+
+          parallel::stopCluster(cl)
+          rm(cl)
+        } else {
+          overlined_simple <- if (requireNamespace("pbapply", quietly = TRUE)) {
+            pbapply::pblapply(sl, function(y) {
+              y <- dplyr::group_by_at(y, attrib)
+              y <- dplyr::summarise(y, do_union = FALSE, .groups = "drop")
+            })
+          } else {
+            lapply(sl, function(y) {
+              y <- dplyr::group_by_at(y, attrib)
+              y <- dplyr::summarise(y, do_union = FALSE, .groups = "drop")
+            })
+          }
+        }
+        rm(sl)
+        overlined_simple <- data.table::rbindlist(overlined_simple)
+        overlined_simple <- sf::st_sf(overlined_simple)
+        overlined_simple <- overlined_simple[seq_len(nrow(overlined_simple)), ]
+      } else {
+        if (!quiet) {
+          message(paste0(Sys.time(), " aggregating flows"))
+        }
+        overlined_simple <- dplyr::group_by_at(sl, attrib)
+        overlined_simple <- dplyr::summarise(overlined_simple, do_union = FALSE, .groups = "drop")
+        rm(sl)
+      }
+
+      overlined_simple <- as.data.frame(overlined_simple)
+      overlined_simple <- sf::st_sf(overlined_simple)
+
+      # Separate our the linestrings and the mulilinestrings
+      if (!quiet) {
+        message(paste0(Sys.time(), " rejoining segments into linestrings"))
+      }
+      overlined_simple <- sf::st_line_merge(overlined_simple)
+      geom_types <- sf::st_geometry_type(overlined_simple)
+      overlined_simple_l <- overlined_simple[geom_types == "LINESTRING", ]
+      overlined_simple_ml <- overlined_simple[geom_types == "MULTILINESTRING", ]
+      suppressWarnings(overlined_simple_ml <-
+        sf::st_cast(
+          sf::st_cast(overlined_simple_ml, "MULTILINESTRING"),
+          "LINESTRING"
+        ))
+
+      return(rbind(overlined_simple_l, overlined_simple_ml))
+    } else {
+      return(sl)
+    }
+  }
+#' @export
+overline.sf <- overline2
+#' @export
+overline.Spatial <- function(sl, ...) {
+  overline_spatial(sl, ...)
 }
+
+#' Spatial aggregation of routes represented with sp classes
+#'
+#' This function, largely superseded by sf implementations, still works
+#' but is not particularly fast.
+#'
+#' @param na.zero Sets whether aggregated values with a value of zero are
+#'   removed.
+#' @inheritParams gsection
+#' @inheritParams overline
+#' @family rnet
 #' @export
-overline.Spatial <- function(sl, attrib, fun = sum, na.zero = FALSE, buff_dist = 0, ...) {
+overline_spatial <- function(sl, attrib, fun = sum, na.zero = FALSE, buff_dist = 0) {
   fun <- c(fun)
   if (length(fun) < length(attrib)) {
     fun <- rep(c(fun), length.out = length(attrib))
@@ -337,196 +544,6 @@ onewaygeo.Spatial <- function(x, attrib) {
   return(singlelines)
 }
 
-#' Convert series of overlapping lines into a route network (new method)
-#'
-#' @description This function is intended as a replacement for overline() and is significantly faster
-#' especially on large datasets. However, it also uses more memory.
-#'
-#' @param sl A spatial object representing routes on a transport network
-#' @param attrib character, column names in sl to be summed
-#' @param ncores integer, how many cores to use in parallel processing, default = 1
-#' @param simplify logical, if TRUE group final segments back into lines, default = TRUE
-#' @param regionalise integer, during simplification regonalisation is used if the number of segments exceeds this value
-#' @param quiet Should the the function omit messages? `NULL` by default,
-#' which means the output will only be shown if `sl` has more than 1000 rows.
-#' @family rnet
-#' @author Malcolm Morgan
-#' @export
-#' @return An `sf` object representing a route network
-#' @rdname overline
-overline2 <- function(sl, attrib, ncores = 1, simplify = TRUE, regionalise = 1e5, quiet = NULL) {
-  if (!"sfc_LINESTRING" %in% class(sf::st_geometry(sl))) {
-    stop("Only LINESTRING is supported")
-  }
-  if (any(c("1", "2", "3", "4", "grid") %in% attrib)) {
-    stop("1, 2, 3, 4, grid are not a permitted column names, please rename that column")
-  }
-  if (is.null(quiet)) {
-    quiet <- ifelse(nrow(sl) < 1000, TRUE, FALSE)
-  }
-  sl <- sf::st_zm(sl)
-  sl <- sl[, attrib]
-  sl_crs <- sf::st_crs(sl)
-  if (!quiet) {
-    message(paste0(Sys.time(), " constructing segments"))
-  }
-  c1 <- sf::st_coordinates(sl)
-  sf::st_geometry(sl) <- NULL
-  l1 <- c1[, 3] # Get which line each point is part of
-  c1 <- c1[, 1:2]
-  l1_start <- duplicated(l1) # find the break points between lines
-  l1_start <- c(l1_start[2:length(l1)], FALSE)
-  c2 <- c1[2:nrow(c1), 1:2] # Create new coords offset by one row
-  c2 <- rbind(c2, c(NA, NA))
-  c2[nrow(c1), ] <- c(NA, NA)
-  c2[!l1_start, 1] <- NA
-  c2[!l1_start, 2] <- NA
-  c3 <- cbind(c1, c2) # make new matrix of start and end coords
-  rm(c1, c2)
-  c3 <- c3[!is.na(c3[, 3]), ]
-  sl <- sl[l1[l1_start], , drop = FALSE] # repeate attributes
-  rm(l1, l1_start)
-
-  # if (!quiet) {
-  #   message(paste0(Sys.time(), " transposing 'B to A' to 'A to B'"))
-  # }
-  attributes(c3)$dimnames <- NULL
-  c3 <- t(apply(c3, MARGIN = 1, FUN = function(y) {
-    if (y[1] != y[3]) {
-      if (y[1] > y[3]) {
-        c(y[3], y[4], y[1], y[2])
-      } else {
-        y
-      }
-    } else {
-      if (y[2] > y[4]) {
-        c(y[3], y[4], y[1], y[2])
-      } else {
-        y
-      }
-    }
-  }))
-
-  # if (!quiet) {
-  #   message(paste0(Sys.time(), " removing duplicates"))
-  # }
-  sl <- cbind(c3, sl)
-  rm(c3)
-
-  sl <- dplyr::group_by_at(sl, c("1", "2", "3", "4"))
-  sl <- dplyr::ungroup(dplyr::summarise_all(sl, .funs = sum))
-  coords <- as.matrix(sl[, 1:4])
-  sl <- sl[, attrib]
-
-  # Make Geometry
-  if (!quiet) {
-    message(paste0(Sys.time(), " building geometry"))
-  }
-  sf::st_geometry(sl) <- sf::st_as_sfc(
-    if (requireNamespace("pbapply", quietly = TRUE)) {
-      pbapply::pblapply(1:nrow(coords), function(y) {
-        sf::st_linestring(matrix(coords[y, ], ncol = 2, byrow = T))
-      })
-    } else {
-      lapply(1:nrow(coords), function(y) {
-        sf::st_linestring(matrix(coords[y, ], ncol = 2, byrow = T))
-      })
-    },
-    crs = sl_crs
-  )
-  rm(coords)
-
-  # Recombine into fewer lines
-  if (simplify) {
-    if (!quiet) {
-      message(paste0(Sys.time(), " simplifying geometry"))
-    }
-    if (nrow(sl) > regionalise) {
-      message(paste0("large data detected, using regionalisation, nrow = ", nrow(sl)))
-      suppressWarnings(cents <- sf::st_centroid(sl))
-      grid <- sf::st_make_grid(cents, what = "polygons")
-      suppressWarnings(inter <- unlist(lapply(sf::st_intersects(cents, grid), `[[`, 1)))
-      sl$grid <- inter
-      rm(cents, grid, inter)
-      # split into a list of df by grid
-      sl <- dplyr::group_split(sl, grid)
-      message(paste0(Sys.time(), " regionalisation complete, aggregating flows"))
-      if (ncores > 1) {
-        cl <- parallel::makeCluster(ncores)
-        parallel::clusterExport(
-          cl = cl,
-          varlist = c("attrib"),
-          envir = environment()
-        )
-        parallel::clusterEvalQ(cl, {
-          library(sf)
-          # library(dplyr)
-        })
-        overlined_simple <- if (requireNamespace("pbapply", quietly = TRUE)) {
-          pbapply::pblapply(sl, function(y) {
-            y <- dplyr::group_by_at(y, attrib)
-            y <- dplyr::summarise(y, do_union = FALSE, .groups = 'drop')
-          }, cl = cl)
-        } else {
-          lapply(sl, function(y) {
-            y <- dplyr::group_by_at(y, attrib)
-            y <- dplyr::summarise(y, do_union = FALSE, .groups = 'drop')
-          })
-        }
-
-
-        parallel::stopCluster(cl)
-        rm(cl)
-      } else {
-        overlined_simple <- if (requireNamespace("pbapply", quietly = TRUE)) {
-          pbapply::pblapply(sl, function(y) {
-            y <- dplyr::group_by_at(y, attrib)
-            y <- dplyr::summarise(y, do_union = FALSE, .groups = 'drop')
-          })
-        } else {
-          lapply(sl, function(y) {
-            y <- dplyr::group_by_at(y, attrib)
-            y <- dplyr::summarise(y, do_union = FALSE, .groups = 'drop')
-          })
-        }
-      }
-      rm(sl)
-      overlined_simple <- data.table::rbindlist(overlined_simple)
-      overlined_simple <- sf::st_sf(overlined_simple)
-      overlined_simple <- overlined_simple[seq_len(nrow(overlined_simple)),]
-    } else {
-      if (!quiet) {
-        message(paste0(Sys.time(), " aggregating flows"))
-      }
-      overlined_simple <- dplyr::group_by_at(sl, attrib)
-      overlined_simple <- dplyr::summarise(overlined_simple, do_union = FALSE, .groups = 'drop')
-      rm(sl)
-    }
-
-    overlined_simple <- as.data.frame(overlined_simple)
-    overlined_simple <- sf::st_sf(overlined_simple)
-
-    # Separate our the linestrings and the mulilinestrings
-    if (!quiet) {
-      message(paste0(Sys.time(), " rejoining segments into linestrings"))
-    }
-    overlined_simple <- sf::st_line_merge(overlined_simple)
-    geom_types <- sf::st_geometry_type(overlined_simple)
-    overlined_simple_l <- overlined_simple[geom_types == "LINESTRING", ]
-    overlined_simple_ml <- overlined_simple[geom_types == "MULTILINESTRING", ]
-    suppressWarnings(overlined_simple_ml <-
-      sf::st_cast(
-        sf::st_cast(overlined_simple_ml, "MULTILINESTRING"),
-        "LINESTRING"
-      ))
-
-    return(rbind(overlined_simple_l, overlined_simple_ml))
-  } else {
-    return(sl)
-  }
-}
-
-
 #' Convert series of overlapping lines into a route network
 #'
 #' This function takes overlapping `LINESTRING`s stored in an
@@ -535,6 +552,7 @@ overline2 <- function(sl, attrib, ncores = 1, simplify = TRUE, regionalise = 1e5
 #'
 #' @param sl An `sf` `LINESTRING` object with overlapping elements
 #' @inheritParams overline
+#' @inheritParams overline_spatial
 #' @export
 #' @examples
 #' routes_fast_sf$value <- 1
@@ -546,7 +564,7 @@ overline2 <- function(sl, attrib, ncores = 1, simplify = TRUE, regionalise = 1e5
 #' sl <- routes_fast_sf[4:7, ]
 #' rnet <- overline_intersection(sl = sl, attrib = c("value", "length"))
 #' plot(rnet, lwd = rnet$value)
-#' rnet_sf <- overline(routes_fast_sf[4:7, ], attrib = c("value", "length"), buff_dist = 10)
+#' rnet_sf <- overline(routes_fast_sf[4:7, ], attrib = c("value", "length"))
 #' plot(rnet_sf, lwd = rnet_sf$value)
 #'
 #' # An even larger example (not shown, takes time to run)
@@ -554,7 +572,7 @@ overline2 <- function(sl, attrib, ncores = 1, simplify = TRUE, regionalise = 1e5
 #' # rnet_sf <- overline(routes_fast_sf, attrib = c("value", "length"), buff_dist = 10)
 #' # plot(rnet$geometry, lwd = rnet$value * 2, col = "grey")
 #' # plot(rnet_sf$geometry,  lwd = rnet_sf$value, add = TRUE)
-overline_intersection <- function(sl, attrib, fun = sum, na.zero = FALSE, buff_dist = 0) {
+overline_intersection <- function(sl, attrib, fun = sum) {
   sl <- sl[attrib]
   sli <- sf::st_intersection(sl)
 
